from __future__ import unicode_literals

from datetime import datetime

import os
import logging


class LogHandle(object):
    __file_dir = None
    __base_name = None

    """
    Logging that becomes easier and more uniformed
    """

    def __init__(self, file_dir=None, base_name=None):
        """
        Logging settings not forgotten!

        :param file_dir: File directory of where log files will be saved
        :param base_name: A base filename that log files will be saved as
        """

        self.file_dir = file_dir
        self.base_name = base_name

    @property
    def file_dir(self):
        return self.__file_dir

    @file_dir.setter
    def file_dir(self, file_dir):
        if not isinstance(file_dir, (str, type(None))):
            raise Exception("'file_dir' is missing")
        if file_dir and not os.path.exists(file_dir):
            raise Exception("'file_dir' does not exist")

        self.__file_dir = file_dir

    @property
    def base_name(self):
        return self.__base_name

    @base_name.setter
    def base_name(self, base_name):
        if not isinstance(base_name, (str, type(None))):
            raise Exception("'base_name' is missing")

        self.__base_name = base_name

    @staticmethod
    def __clean_handlers():
        for handler in logging.root.handlers[:]:
            logging.root.removeHandler(handler)

    def write_to_log(self, message, action='info'):
        """
        Logging becomes more organized!

        :param message: Message that you would like to write to log
        :param action: What kind of action would you like the entry to be marked as?
            (debug, info, warning, error, critical)
        """
        if not message:
            raise Exception("'message' is missing")

        self.__clean_handlers()
        filepath = os.path.join(self.__file_dir,
                                "{0}_{1}_Log.txt".format(datetime.now().__format__("%Y%m%d"), self.__base_name))

        logging.basicConfig(filename=filepath,
                            level=logging.DEBUG, format=' %(asctime)s - %(levelname)s - %(message)s')

        print('{0} - {1} - {2}'.format(datetime.now(), action.upper(), message))

        if action.lower() == 'debug':
            logging.debug(message)
        elif action.lower() == 'info':
            logging.info(message)
        elif action.lower() == 'warning':
            logging.warning(message)
        elif action.lower() == 'error':
            logging.error(message)
        elif action.lower() == 'critical':
            logging.critical(message)

    def __del__(self):
        logging.shutdown()



from getopt import GetoptError
import sys

if MYPY_CHECK_RUNNING:
    from typing import List, Optional

logger = logging.getLogger(__name__)


def main(args=None):
    # type: (Optional[List[str]]) -> int
    if args is None:
        args = sys.argv[1:]

    try:
        opts, args = parse_command(args)
    except GetoptError as exc:
        sys.stderr.write("ERROR: %s" % exc)
        sys.stderr.write(os.linesep)
        return 1

    for cmd, arg in opts:
        if cmd == '-h':
            print('KGlobal -create_master_salt')
        elif cmd == '-create_master_salt':
            try:
                print('Creating Master Salt Key....')
                from . import create_master_salt_key
                create_master_salt_key()
                print('Master Salt Key has been successfully created')
            except Exception as exc:
                sys.stderr.write("ERROR: %s" % exc)
                sys.stderr.write(os.linesep)
                return 1

    return 0



from __future__ import absolute_import

import os
import sys

if __package__ == '':
    path = os.path.dirname(os.path.dirname(__file__))
    sys.path.insert(0, path)

from .main import main as _main

if __name__ == '__main__':
    sys.exit(_main())



from __future__ import unicode_literals

import pickle
import os


def create_salt(salt_dir, salt_fn):
    """
    Create salt the way I like it. Salty!

    :param salt_dir: Directory for where the salt key file will be created
    :param salt_fn: File name for what the salt key file will be saved as
    """

    try:
        from . import file_write_bytes, SaltHandle
        path = os.path.join(salt_dir, salt_fn)

        if not os.path.exists(salt_dir) or not os.path.exists(path):
            if not os.path.exists(salt_dir):
                os.makedirs(salt_dir)

            salt_key = SaltHandle()
            file_write_bytes(path, pickle.dumps(salt_key))
    except ImportError:
        raise



__package_name__ = 'KGlobal'
__author__ = 'Kevin Russell'
__version__ = "1.0.15"
__description__ = '''File, encryption, SQL, XML, and etc...'''
__url__ = 'https://github.com/KLRussell/Python_KGlobal_Package'


if __name__ == '__main__':
    from runpy import run_module
    run_module('KGlobal_Core')
